use serde::{Deserialize, Serialize};
use serde_json;

use crate::database;
use crate::error::*;

pub fn endpoint(endpoint: &str, input: serde_json::Value) -> Result<serde_json::Value, Error> {
    match endpoint {
        "echo" => Ok(serde_json::to_value(echo(serde_json::from_value(input)?)?)?),
        _ => Err(Error::new(Kind::InvalidEndpointError, Class::ClientError)),
    }
}

#[derive(Serialize, Deserialize)]
pub struct EchoInput {
    message: String,
}

#[derive(Serialize, Deserialize)]
pub struct EchoOutput {
    message: String,
    database: Option<i32>,
}

pub fn echo(input: EchoInput) -> Result<EchoOutput, Error> {
    let backend = database::client()?;
    let mut output = EchoOutput {
        message: input.message,
        database: None,
    };
    match backend {
        database::Client::Postgres(mut client) => {
            let res = client.query("SELECT * FROM test", &[])?;
            for row in res {
                output.database = Some(row.get(0));
            }
        }
    }
    Ok(output)
}
